@@ -31,11 +31,12 @@
 #import "DBCVSReader.h"
 #import "DBLogger.h"
 #import "DBProgress.h"
+#import "Preferences.h"
 
 #define DB_ENVIRONMENT_PRODUCTION 0
 #define DB_ENVIRONMENT_SANDBOX    1
 
-#define MAX_STORED_LOGINS 5
+#define MAX_STORED_LOGINS 10
 
 #if defined(__APPLE__) && (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
 #define NSUTF16StringEncoding 999
@@ -63,20 +64,12 @@
 {
   if ((self = [super init]))
     {
-      id logLevelObj;
       NSUserDefaults *defaults;
 
       defaults = [NSUserDefaults standardUserDefaults];
 
       logger = [[DBLogger alloc] init];
-      logLevelObj = [defaults objectForKey: @"LogLevel"];
-      /* if the log level is not set we set it to the standard level */
-      if (logLevelObj == nil)
-        {
-          logLevelObj = [NSNumber numberWithInt: LogStandard];
-          [defaults setObject:logLevelObj forKey: @"LogLevel"];
-        }
-      [logger setLogLevel: [logLevelObj intValue]];
+      [self reloadDefaults];
 
       loginDict = [defaults objectForKey: @"logins"];
       if (loginDict == nil)
@@ -84,12 +77,18 @@
       else
 	loginDict = [NSMutableDictionary dictionaryWithDictionary:loginDict];
       [loginDict retain];
+
+      /* if none found, set a reasonable default for the upload and insert batch size */
+      if (![defaults objectForKey:@"UpBatchSize"])
+	[defaults setObject:[NSNumber numberWithInt:100] forKey:@"UpBatchSize"];
+
     }
   return self;
 }
 
 - (void)dealloc
 {
+  [preferences release];
   [dbCsv release];
   [db release];
   [logger release];
@@ -98,12 +97,7 @@
 }
 
 - (void)awakeFromNib
-{
-#if defined(__APPLE__) && (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
-  [popupStrEncoding setAutoenablesItems: NO];
-  [[popupStrEncoding itemAtIndex: 1] setEnabled: NO];
-#endif
-  
+{  
   objInspector = [[DBObjectInspector alloc] init];
 }
 
@@ -125,100 +119,49 @@
   return NO;
 }
 
+- (void)reloadDefaults
+{
+  NSUserDefaults *defaults;
+  id obj;
+  
+  defaults = [NSUserDefaults standardUserDefaults];
+  
+  obj = [defaults objectForKey: @"LogLevel"];
+  /* if the log level is not set we set it to the standard level */
+  if (obj == nil)
+    {
+      obj = [NSNumber numberWithInt: LogStandard];
+      [defaults setObject:obj forKey: @"LogLevel"];
+    }
+
+  [logger setLogLevel: [obj intValue]];
+
+  obj = [defaults objectForKey:@"UpBatchSize"];
+  if (obj)
+    {
+      int size;
+
+      size = [obj intValue];
+      if (size > 0)
+	[db setUpBatchSize:size];
+    }
+}
+
 - (IBAction)showPrefPanel:(id)sender
 {
   NSUserDefaults *defaults;
-  int index;
-
-  defaults = [NSUserDefaults standardUserDefaults];
   
-  index = 0;
-  switch([[defaults valueForKey: @"StringEncoding"] intValue])
+  if (!preferences)
     {
-      case NSUTF8StringEncoding:
-        index = 0;
-        break;
-      case NSUTF16StringEncoding:
-        index = 1;
-        break;
-      case NSISOLatin1StringEncoding:
-        index = 2;
-        break;
-      case NSWindowsCP1252StringEncoding:
-        index = 3;
-        break;
+      preferences = [[Preferences alloc] init];
+      [preferences setAppController:self];
     }
-  [popupStrEncoding selectItemAtIndex: index];
-
-  [buttPrefHttps setState: [[defaults valueForKey: @"UseHttps"] intValue]];
-
-  index = 0;
-  switch([[defaults valueForKey: @"LogLevel"] intValue])
-    {
-      case LogStandard:
-        index = 0;
-        break;
-      case LogInformative:
-        index = 1;
-        break;
-      case LogDebug:
-        index = 2;
-        break;
-      default:
-        NSLog(@"Unexpected log level");
-        break;
-    }
-  [popupLogLevel selectItemAtIndex: index];
-
-  [prefPanel makeKeyAndOrderFront: sender];
-}
-
-- (IBAction)prefPanelCancel:(id)sender
-{
-  [prefPanel performClose: nil];
-}
-
-- (IBAction)prefPanelOk:(id)sender
-{
-  NSStringEncoding selectedEncoding;
-  DBLogLevel selectedLogLevel;
-  NSUserDefaults *defaults;
-
-  defaults = [NSUserDefaults standardUserDefaults];
   
-  selectedEncoding = NSUTF8StringEncoding;
-  switch([popupStrEncoding indexOfSelectedItem])
-    {
-      case 0: selectedEncoding = NSUTF8StringEncoding;
-        break;
-      case 1: selectedEncoding = NSUTF16StringEncoding;
-        break;
-      case 2: selectedEncoding = NSISOLatin1StringEncoding;
-        break;
-      case 3: selectedEncoding = NSWindowsCP1252StringEncoding;
-        break;
-    }
-    
-  [defaults setObject:[NSNumber numberWithInt: selectedEncoding] forKey: @"StringEncoding"];
+  [preferences showPrefPanel:sender];
 
-  [defaults setObject:[NSNumber numberWithInt: [buttPrefHttps state]] forKey: @"UseHttps"];
-
-  selectedLogLevel = LogStandard;
-  switch([popupLogLevel indexOfSelectedItem])
-    {
-      case 0: selectedLogLevel = LogStandard;
-        break;
-      case 1: selectedLogLevel = LogInformative;
-        break;
-      case 2: selectedLogLevel = LogDebug;
-        break;
-      default:
-        break;
-    }
-  [defaults setObject:[NSNumber numberWithInt: selectedLogLevel] forKey: @"LogLevel"];
-  [logger setLogLevel: selectedLogLevel];
-
-  [prefPanel performClose: nil];
+  /* Apply defaults */
+  defaults = [NSUserDefaults standardUserDefaults];
+  [logger setLogLevel: [[defaults valueForKey: @"StringEncoding"] intValue]];
 }
 
 
@@ -339,6 +282,7 @@
 
   NS_HANDLER
     [logger log:LogStandard :@"Login failed\n"];
+  [loginStatus setImage:[NSImage imageNamed:@"butt_red_16.tif"]];
     if ([[localException name] hasPrefix:@"DB"])
       {
         [faultTextView setString:[localException reason]];
@@ -352,6 +296,7 @@
   NS_ENDHANDLER
   [logger log:LogStandard :@"[AppController doLogin] %@ logged in succesfully\n", userName];
   
+  [loginStatus setImage:[NSImage imageNamed:@"butt_green_16.tif"]];
   loginSet = [NSMutableDictionary dictionaryWithCapacity:4];
   [loginSet retain];
   [loginSet setObject:userName forKey:@"username"];
@@ -409,6 +354,7 @@
 
 - (IBAction)showSelect:(id)sender
 {
+  [progIndSelect setDoubleValue:0];
   [winSelect makeKeyAndOrderFront:self];
 }
 
@@ -453,6 +399,8 @@
     }
   
   progress = [[DBProgress alloc] init];
+  [progress setProgressIndicator: progIndSelect];
+  [progress setRemainingTimeField: fieldRTSelect];
   [progress setLogger:logger];
   [progress reset];
   cvsWriter = [[DBCVSWriter alloc] initWithHandle:fileHandle];
@@ -479,6 +427,8 @@
 {
   NSArray *objectNames;
 
+  [winInsert makeKeyAndOrderFront:self];
+  [progIndInsert setIndeterminate:YES];
   objectNames = nil;
   NS_DURING
     objectNames = [db sObjectNames];
@@ -491,8 +441,8 @@
   NS_ENDHANDLER
   [popupObjectsInsert removeAllItems];
   [popupObjectsInsert addItemsWithTitles: objectNames];
-
-  [winInsert makeKeyAndOrderFront:self];
+  [progIndInsert setIndeterminate:NO];
+  [progIndInsert setDoubleValue:0];
 }
 
 - (IBAction)browseFileInsert:(id)sender
@@ -512,25 +462,39 @@
 
 - (IBAction)executeInsert:(id)sender
 {
-  NSString      *filePath;
-  DBCVSReader   *reader;
-  NSString      *intoWhichObject;
-  DBProgress    *progress;
+  NSString       *filePath;
+  NSString       *resFilePath;
+  DBCVSReader    *reader;
+  NSString       *intoWhichObject;
+  DBProgress     *progress;
+  NSMutableArray *results;
+  NSFileManager  *fileManager;
+  NSFileHandle   *resFH;
+  NSUserDefaults *defaults;
+  DBCVSWriter    *resWriter;
   
+  defaults = [NSUserDefaults standardUserDefaults];  
+  filePath = [fieldFileInsert stringValue];
+  resFilePath = [[filePath stringByDeletingLastPathComponent] stringByAppendingPathComponent: @"results.csv"];
+
+  NSLog(@"writing results to: %@", resFilePath);
+
   filePath = [fieldFileInsert stringValue];
-  NSLog(@"%@", filePath);
   
   intoWhichObject = [[[popupObjectsInsert selectedItem] title] retain];
   [logger log:LogInformative :@"[AppController executeInsert] object: %@\n", intoWhichObject];
 
   progress = [[DBProgress alloc] init];
   [progress setLogger:logger];
+  [progress setProgressIndicator: progIndInsert];
+  [progress setRemainingTimeField: fieldRTInsert];
   [progress reset];
   
+  results = nil;
   reader = [[DBCVSReader alloc] initWithPath:filePath withLogger:logger];
-  
   NS_DURING
-    [dbCsv create:intoWhichObject fromReader:reader progressMonitor:progress];
+    results = [dbCsv create:intoWhichObject fromReader:reader progressMonitor:progress];
+    [results retain];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
@@ -538,10 +502,35 @@
         [faultPanel makeKeyAndOrderFront:nil];
       }
   NS_ENDHANDLER
-  
+
+
+  fileManager = [NSFileManager defaultManager];
+  if ([fileManager createFileAtPath:resFilePath contents:nil attributes:nil] == NO)
+    {
+      NSRunAlertPanel(@"Attention", @"Could not create File.", @"Ok", nil, nil);
+    }  
+
+  resFH = [NSFileHandle fileHandleForWritingAtPath:resFilePath];
+  if (resFH == nil)
+    {
+      NSRunAlertPanel(@"Attention", @"Cannot create File.", @"Ok", nil, nil);
+    }
+  else
+    {
+      resWriter = [[DBCVSWriter alloc] initWithHandle:resFH];
+      [resWriter setLogger:logger];
+      [resWriter setStringEncoding: [[defaults valueForKey: @"StringEncoding"] intValue]];
+      
+      [resWriter setFieldNames:[results objectAtIndex: 0] andWriteIt:YES];
+      [resWriter writeDataSet: results];
+      
+      [resWriter release];
+    }
+
   [reader release];
   [intoWhichObject release];
   [progress release];
+  [results release];
 }
 
 /* UPDATE */
@@ -551,6 +540,8 @@
 {
   NSArray      *objectNames;
   
+  [winUpdate makeKeyAndOrderFront:self];
+  [progIndUpdate setIndeterminate:YES];
   objectNames  = nil;
   NS_DURING
     objectNames = [db sObjectNames];
@@ -563,8 +554,8 @@
   NS_ENDHANDLER
   [popupObjectsUpdate removeAllItems];
   [popupObjectsUpdate addItemsWithTitles: objectNames];
-    
-  [winUpdate makeKeyAndOrderFront:self];
+  [progIndUpdate setIndeterminate:NO];
+  [progIndUpdate setDoubleValue:0]; 
 }
 
 - (IBAction)browseFileUpdate:(id)sender
@@ -584,25 +575,37 @@
 
 - (IBAction)executeUpdate:(id)sender
 {
-  NSString      *filePath;
-  DBCVSReader   *reader;
-  NSString      *whichObject;
-  DBProgress    *progress;
-  
+  NSString       *filePath;
+  NSString       *resFilePath;
+  DBCVSReader    *reader;
+  NSString       *whichObject;
+  DBProgress     *progress;
+  NSMutableArray *results;
+  NSFileManager  *fileManager;
+  NSFileHandle   *resFH;
+  NSUserDefaults *defaults;
+  DBCVSWriter    *resWriter;
+
+  defaults = [NSUserDefaults standardUserDefaults];  
   filePath = [fieldFileUpdate stringValue];
-  NSLog(@"%@", filePath);
+  resFilePath = [[filePath stringByDeletingLastPathComponent] stringByAppendingPathComponent: @"results.csv"];
+
+  [logger log:LogDebug: @"[AppController executeUpdate] writing results to: %@", resFilePath];
   
   progress = [[DBProgress alloc] init];
   [progress setLogger:logger];
+  [progress setProgressIndicator: progIndUpdate];
+  [progress setRemainingTimeField: fieldRTUpdate];
   [progress reset];
 
   whichObject = [[[popupObjectsUpdate selectedItem] title] retain];
   [logger log:LogInformative :@"[AppController executeUpdate] object: %@\n", whichObject];
   
+  results = nil;
   reader = [[DBCVSReader alloc] initWithPath:filePath withLogger:logger];
-  
   NS_DURING
-    [dbCsv update:whichObject fromReader:reader progressMonitor:progress];
+    results = [dbCsv update:whichObject fromReader:reader progressMonitor:progress];
+    [results retain];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
@@ -610,10 +613,34 @@
         [faultPanel makeKeyAndOrderFront:nil];
       }
   NS_ENDHANDLER
+
+  fileManager = [NSFileManager defaultManager];
+  if ([fileManager createFileAtPath:resFilePath contents:nil attributes:nil] == NO)
+    {
+      NSRunAlertPanel(@"Attention", @"Could not create File.", @"Ok", nil, nil);
+    }  
+
+  resFH = [NSFileHandle fileHandleForWritingAtPath:resFilePath];
+  if (resFH == nil)
+    {
+      NSRunAlertPanel(@"Attention", @"Cannot create File.", @"Ok", nil, nil);
+    }
+  else
+    {
+      resWriter = [[DBCVSWriter alloc] initWithHandle:resFH];
+      [resWriter setLogger:logger];
+      [resWriter setStringEncoding: [[defaults valueForKey: @"StringEncoding"] intValue]];
+      
+      [resWriter setFieldNames:[results objectAtIndex: 0] andWriteIt:YES];
+      [resWriter writeDataSet: results];
+      
+      [resWriter release];
+    }
     
   [reader release];
   [whichObject release];
   [progress release];
+  [results release];
 }
 
 /*  SELECT IDENTIFY */
@@ -621,6 +648,8 @@
 - (IBAction)showSelectIdentify:(id)sender
 {
   [winSelectIdentify makeKeyAndOrderFront:self];
+  [progIndSelectIdent setIndeterminate:NO];
+  [progIndSelectIdent setDoubleValue:0];
 }
 
 - (IBAction)browseFileSelectIdentifyIn:(id)sender
@@ -711,6 +740,8 @@
   [cvsWriter setLogger:logger];
   progress = [[DBProgress alloc] init];
   [progress setLogger:logger];
+  [progress setProgressIndicator: progIndSelectIdent];
+  [progress setRemainingTimeField: fieldRTSelectIdent];
   [progress reset];
 
   NS_DURING
@@ -833,7 +864,7 @@
   
   NS_DURING
     [fieldStatusQd setStringValue:@"Working..."];
-    resultArray = [db delete: idArray];
+    resultArray = [db delete: idArray progressMonitor:nil];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
@@ -860,11 +891,11 @@
       }
     else
       {
-	NSDictionary *resultDict;
-	NSString     *resultMsgStr;
+        NSDictionary *resultDict;
+        NSString     *resultMsgStr;
 
-	resultDict = [resultArray objectAtIndex:0];
-	resultMsgStr = [resultDict objectForKey:@"message"];
+        resultDict = [resultArray objectAtIndex:0];
+        resultMsgStr = [resultDict objectForKey:@"message"];
         [fieldStatusQd setStringValue:[resultDict objectForKey:@"statusCode"]];
         [faultTextView setString:resultMsgStr];
         [faultPanel makeKeyAndOrderFront:nil];
@@ -876,6 +907,8 @@
 - (IBAction)showDelete:(id)sender
 {
   [winDelete makeKeyAndOrderFront:self];
+  [progIndDelete setIndeterminate:NO];
+  [progIndDelete setDoubleValue:0];
 }
 
 - (IBAction)browseFileDelete:(id)sender
@@ -895,15 +928,34 @@
 
 - (IBAction)executeDelete:(id)sender
 {
-  NSString      *filePath;
-  DBCVSReader   *reader;
-  
+  NSString       *filePath;
+  NSString       *resFilePath;
+  DBCVSReader    *reader;
+  DBCVSWriter    *resWriter;
+  NSMutableArray *results;
+  NSFileManager  *fileManager;
+  NSFileHandle   *resFH;
+  NSUserDefaults *defaults;
+  DBProgress     *progress;
+
+  defaults = [NSUserDefaults standardUserDefaults];  
   filePath = [fieldFileDelete stringValue];
+  resFilePath = [[filePath stringByDeletingLastPathComponent] stringByAppendingPathComponent: @"results.csv"];
+
+  NSLog(@"writing results to: %@", resFilePath);
     
-  reader = [[DBCVSReader alloc] initWithPath:filePath byParsingHeaders:([checkSkipFirstLine state]==NSOnState) withLogger:logger];  
-  
+  reader = [[DBCVSReader alloc] initWithPath:filePath byParsingHeaders:([checkSkipFirstLine state]==NSOnState) withLogger:logger];
+
+  progress = [[DBProgress alloc] init];
+  [progress setProgressIndicator: progIndDelete];
+  [progress setRemainingTimeField: fieldRTDelete];
+  [progress setLogger:logger];
+  [progress reset];
+
+  results = nil;  
   NS_DURING
-    [dbCsv deleteFromReader:reader];
+    results = [dbCsv deleteFromReader:reader progressMonitor:progress];
+    [results retain];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
@@ -912,7 +964,31 @@
       }
   NS_ENDHANDLER
 
+  fileManager = [NSFileManager defaultManager];
+  if ([fileManager createFileAtPath:resFilePath contents:nil attributes:nil] == NO)
+    {
+      NSRunAlertPanel(@"Attention", @"Could not create File.", @"Ok", nil, nil);
+    }  
+
+  resFH = [NSFileHandle fileHandleForWritingAtPath:resFilePath];
+  if (resFH == nil)
+    {
+      NSRunAlertPanel(@"Attention", @"Cannot create File.", @"Ok", nil, nil);
+    }
+  else
+    {
+      resWriter = [[DBCVSWriter alloc] initWithHandle:resFH];
+      [resWriter setLogger:logger];
+      [resWriter setStringEncoding: [[defaults valueForKey: @"StringEncoding"] intValue]];
+      
+      [resWriter setFieldNames:[results objectAtIndex: 0] andWriteIt:YES];
+      [resWriter writeDataSet: results];
+      
+      [resWriter release];
+    }
+  [results release];
   [reader release];
+  [progress release];
 }
 
 /* OBJECT INSPECTOR */
