@@ -1,25 +1,26 @@
 /*
-   Project: DataBasin
+  Project: DataBasin
 
-   Copyright (C) 2009-2013 Free Software Foundation
+  Copyright (C) 2009-2014 Free Software Foundation
 
-   Author: Riccardo Mottola
+  Author: Riccardo Mottola
 
-   Created: 2009-06-24 22:34:06 +0200 by multix
+  Created: 2009-06-24 22:34:06 +0200 by multix
 
-   This application is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+  This library is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2 of the License, or (at your option) any later version.
 
-   This application is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+  This library is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Library General Public License for more details.
 
-   You should have received a copy of the GNU General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+  You should have received a copy of the GNU Lesser General Public
+  License along with this library; if not, write to the Free
+  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+  Boston, MA 02111 USA.
 */
 
 #import "DBCVSReader.h"
@@ -182,7 +183,7 @@
 	  if (loc == [line length]-1 && token == nil)
 	    {
 	      token = @"";
-	      NSLog(@"we reached the end... token is %@", token);
+              //	      NSLog(@"we reached the end... token is %@", token);
 	    }
 	  //NSLog(@"loc %lu", loc);
 	  if (loc > 0 && token)
