@@ -1,7 +1,7 @@
 /*
    Project: DataBasin
 
-   Copyright (C) 2009-2013 Free Software Foundation
+   Copyright (C) 2009-2014 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -176,12 +176,16 @@
       
       if ([obj isKindOfClass: [NSDictionary class]])
         {
-	  NSMutableString *s;
+          NSMutableString *s;
 
-	  s = [NSMutableString stringWithString:root];
+          if (root)
+            s = [NSMutableString stringWithString:root];
+          else
+            s = [NSMutableString stringWithString:@""];
 	  if (headerFlag)
 	    {
-	      [s appendString:@"."];
+              if (root)
+                [s appendString:@"."];
 	      [s appendString:key];
 	    }
 
@@ -191,10 +195,14 @@
         {
 	  NSMutableString *s;
 
+          if (root)
+            s = [NSMutableString stringWithString:root];
+          else
+            s = [NSMutableString stringWithString:@""];
 	  if (headerFlag)
 	    {
-	      s = [NSMutableString stringWithString:root];
-	      [s appendString:@"."];
+              if (root)
+                [s appendString:@"."];
 	      [s appendString:key];
 	    }
 	  else
@@ -212,15 +220,16 @@
 
 - (void)setFieldNames:(id)obj andWriteIt:(BOOL)flag
 {
-  /* if we write the header, fine, else we write at least the BOM */
+  /* if we have no data, we return */
+  if (obj == nil)
+    return;
 
+  /* if we write the header, fine, else we write at least the BOM */
   if (flag == YES)
     {
       NSString *theLine;
       NSArray *array;
 
-      NSLog(@"should write out field names to file");
-
       /* if we have just a single object, we fake an array */
       if([obj isKindOfClass: [NSArray class]])
 	array = obj;
@@ -268,16 +277,23 @@
     }
 }
 
-- (NSString *)formatOneLine:(NSArray *)values forHeader:(BOOL) headerFlag
+- (NSString *)formatOneLine:(id)data forHeader:(BOOL) headerFlag
 {
+  NSArray         *array;
   NSMutableString *theLine;
-  unsigned      size;
-  unsigned      i;
-  id       obj;
+  unsigned         size;
+  unsigned         i;
+  id               obj;
   
-//  NSLog(@"Format one line array: %@", values);
 
-  size = [values count];
+
+  /* if we have just a single object, we fake an array */
+  if([data isKindOfClass: [NSArray class]])
+    array = data;
+  else
+    array = [NSArray arrayWithObject: data];
+
+  size = [array count];
 
   if (size == 0)
     return nil;
@@ -286,17 +302,15 @@
 
   for (i = 0; i < size; i++)
     {
-      obj = [values objectAtIndex:i];
+      obj = [array objectAtIndex:i];
       if ([obj isKindOfClass: [NSDictionary class]])
         {
-	  //          NSLog(@"Dictionary: %@", obj);
 	  [theLine appendString: [self formatComplexObject: obj withRoot: nil forHeader:headerFlag]];
         }
-      if ([obj isKindOfClass: [DBSObject class]])
+      else if ([obj isKindOfClass: [DBSObject class]])
         {
 	  NSArray *keys;
 	  unsigned j;
-	  NSMutableArray *values;
 
 	  keys = [obj fieldNames];
 
@@ -306,7 +320,7 @@
 	      id value;
 
 	      key = [keys objectAtIndex: j];
-	      value = [obj fieldValue: key];
+	      value = [obj valueForField: key];
 	      // NSLog(@"key ---> %@ object %@", key, value);
 	      
 	      if ([value isKindOfClass: [NSString class]] ||[value isKindOfClass: [NSNumber class]] )
