@@ -2,26 +2,28 @@
  Project: DataBasin
  DBSobject.m
  
- Copyright (C) 2010-2013 Free Software Foundation
+ Copyright (C) 2010-2014 Free Software Foundation
  
  Author: Riccardo Mottola
  
  Created by Riccardo Mottola on 20/07/10.
  
- This application is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public
+
+ This library is free software; you can redistribute it and/or
+ modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
- 
- This application is distributed in the hope that it will be useful,
+
+ This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
- 
- You should have received a copy of the GNU General Public
+
+ You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free
- Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
- */
+ Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ Boston, MA 02111 USA.
+*/
 
 
 #import "DBSObject.h"
@@ -72,15 +74,38 @@
   return self;
 }
 
+- (id)copyWithZone: (NSZone *)zone
+{
+  DBSObject *sobj;
+
+  sobj = [[[self class] allocWithZone: zone] init];
+  if(sobj)
+    {
+      sobj->dbs = dbs;
+      sobj->fieldNames = [fieldNames mutableCopy];
+      sobj->fieldProperties = [fieldProperties mutableCopy];
+      sobj->recordValues = [recordValues mutableCopy];
+      sobj->objectProperties = [objectProperties mutableCopy];
+      sobj->recordTypes = [recordTypes copy];
+    }
+  return sobj;
+}
+
 - (void)dealloc
 {
   [fieldNames release];
   [fieldProperties release];
   [recordValues release];
   [objectProperties release];
+  [recordTypes release];
   [super dealloc];
 }
 
+- (NSString *)description
+{
+  return [NSString stringWithFormat: @"<DBSObject %@, %@, %@>", [self name], [self sfId], [self label]];
+}
+
 /** <p>returns the current salesforce id, in whichever format it is currently stored.</p>
 */
 - (NSString *)sfId
@@ -146,9 +171,41 @@ converting it if necessary.</p>
 
 - (NSString *)name
 {
+  /*
+  NSString *name;
+
+  name = nil;
+  if ([[self type] isEqualToString:@"Contract"])
+    name = [recordValues objectForKey: @"ContractNumber"];
+  else if ([[self type] isEqualToString:@"Case"])
+    name = [recordValues objectForKey: @"CaseNumber"];
+  else
+    name = [recordValues objectForKey: @"Name"];
+
+  if (name == nil)
+    name = [objectProperties objectForKey: @"name"];
+
+  return name;
+  */
   return [objectProperties objectForKey: @"name"];
 }
 
+- (NSString *)type
+{
+  return [objectProperties objectForKey: @"type"];
+}
+
+- (NSString *)label
+{
+  return [objectProperties objectForKey: @"label"];
+}
+
+- (NSString *)keyPrefix
+{
+  return [objectProperties objectForKey: @"keyPrefix"];
+}
+
+
 - (void)setProperties: (NSDictionary *)properties forField: (NSString *)field
 {
   // TODO should check if field is not already present
@@ -176,7 +233,7 @@ converting it if necessary.</p>
 
 /** <p>Returns the value of field <i>field</i><p>
 */
-- (id)fieldValue: (NSString *)field
+- (id)valueForField: (NSString *)field
 {
   return [recordValues objectForKey: field];
 }
@@ -192,6 +249,17 @@ converting it if necessary.</p>
   [recordValues setObject: value forKey: field];
 }
 
+- (void)setRecordTypes: (NSArray *)rtInfo
+{
+  ASSIGN(recordTypes, rtInfo);
+}
+
+- (NSArray *)recordTypes
+{
+  return recordTypes;
+}
+
+
 - (void)loadFieldValues
 {
   unsigned int i;
@@ -269,7 +337,7 @@ converting it if necessary.</p>
       NSString *fieldName;
 
       fieldName = [namesArray objectAtIndex: i];
-      [self setValue: [tempObj fieldValue: fieldName] forField: fieldName];
+      [self setValue: [tempObj valueForField: fieldName] forField: fieldName];
     }
 }
 
