@@ -1,33 +1,32 @@
 /*
-   Project: DataBasin
+  Project: DataBasin
 
-   Copyright (C) 2008-2013 Free Software Foundation
+  Copyright (C) 2008-2014 Free Software Foundation
 
-   Author: Riccardo Mottola
+  Author: Riccardo Mottola
 
-   Created: 2008-11-13 22:44:45 +0100 by multix
+  Created: 2008-11-13 22:44:45 +0100 by multix
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
- 
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
- 
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
- */
+  This library is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2 of the License, or (at your option) any later version.
+
+  This library is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Library General Public License for more details.
+
+  You should have received a copy of the GNU Lesser General Public
+  License along with this library; if not, write to the Free
+  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+  Boston, MA 02111 USA.
+*/
 
 #import <AppKit/AppKit.h>
 
 #import "DBSoap.h"
 #import "DBSObject.h"
-#import "DBCVSWriter.h"
-#import "DBCVSReader.h"
 
 #import "DBProgressProtocol.h"
 
@@ -37,8 +36,24 @@
 {
   if ((self = [super init]))
     {
+      NSUserDefaults *defaults;
+      id obj;
+
+      defaults = [NSUserDefaults standardUserDefaults];
+      
       standardTimeoutSec = 60;
       queryTimeoutSec = 180;
+      
+      upBatchSize = 1;
+      obj = [defaults objectForKey:@"UpBatchSize"];
+      if (obj)
+	{
+	  int size;
+	  
+	  size = [obj intValue];
+	  if (size > 0)
+	    upBatchSize = size;
+	}
     }
   return self;
 }
@@ -55,6 +70,11 @@
   return logger;
 }
 
+- (void)setUpBatchSize:(unsigned)size
+{
+  upBatchSize = size;
+}
+
 /**<p>executes login</p>
    <p><i>url</i> specifies the URL of the endpoint</p>
    <p><i>useHttps</i> specifies if secure connecton has to be used or not. If not, http is attempted and then enforced.
@@ -358,12 +378,13 @@
   
   if (sizeStr != nil)
     {
-      unsigned     i;
-      unsigned     j;
-      int    batchSize;
+      NSUInteger    i;
+      NSUInteger    j;
+      NSUInteger    batchSize;
       NSMutableArray *keys;
       NSScanner *scan;
       long long ll;
+      BOOL typePresent;
 
       scan = [NSScanner scannerWithString:sizeStr];
       if ([scan scanLongLong:&ll])
@@ -403,7 +424,9 @@
       [keys removeObject:@"GWSCoderOrder"];
       
       /* remove some fields which get added automatically by salesforce even if not asked for */
-      [keys removeObject:@"type"];
+      typePresent = [keys containsObject:@"type"];
+      if (typePresent)
+        [keys removeObject:@"type"];
       
       /* remove Id only if it is null, else an array of two populated Id is returned by SF */
       if (![[record objectForKey:@"Id"] isKindOfClass: [NSArray class]])
@@ -445,6 +468,15 @@
                 value = obj;
               [sObj setValue: value forField: key];
             }
+
+          /* we removed type, but if it is present, set it as a property */
+          if (typePresent) 
+            {
+              NSDictionary *propDict;
+
+              propDict = [NSDictionary dictionaryWithObject:[record objectForKey: @"type"] forKey:@"type"];
+              [sObj setObjectProperties: propDict];
+            }
           [objects addObject:sObj];
           [sObj release];
         }
@@ -549,10 +581,11 @@
 
   if (sizeStr != nil)
     {
-      int            i;
-      int            j;
+      NSUInteger     i;
+      NSUInteger     j;
       NSUInteger     batchSize;
       NSMutableArray *keys;
+      BOOL typePresent;
       
       /* this will be only as big as a batch anyway */
       size = (unsigned long)[sizeStr intValue];
@@ -573,7 +606,9 @@
       [keys removeObject:@"GWSCoderOrder"];
 
       /* remove some fields which get added automatically by salesforce even if not asked for */
-      [keys removeObject:@"type"];
+      typePresent = [keys containsObject:@"type"];
+      if (typePresent)
+        [keys removeObject:@"type"];
       
       /* remove Id only if it is null, else an array of two populated Id is returned by SF */
       if (![[record objectForKey:@"Id"] isKindOfClass: [NSArray class]])
@@ -604,6 +639,16 @@
                   value = obj;
               [sObj setValue: value forField: key];
             }
+
+          /* we removed type, but if it is present, set it as a property */
+          if (typePresent) 
+            {
+              NSDictionary *propDict;
+              
+              propDict = [NSDictionary dictionaryWithObject:[record objectForKey: @"type"] forKey:@"type"];
+              [sObj setObjectProperties: propDict];
+            }
+
           [objects addObject:sObj];
           [sObj release];
         }
@@ -639,8 +684,10 @@
   BOOL multiKey;
   NSString *identifier;
   NSString *queryFirstPart;
-  NSString *queryLimitPart;
+  NSString *queryOptionsPart;
   NSUInteger limitLocation;
+  NSUInteger orderByLocation;
+  NSUInteger optionsLocation;
 
   multiKey = NO;
   identifier = nil;
@@ -670,16 +717,41 @@
    else if (batchSize > 1)
      batchable = YES;
   
+  optionsLocation = NSNotFound;
   limitLocation = [queryString rangeOfString: @"LIMIT" options:NSCaseInsensitiveSearch].location;
-  if (limitLocation != NSNotFound)
+  orderByLocation = [queryString rangeOfString: @"ORDER BY" options:NSCaseInsensitiveSearch].location;
+  if (limitLocation != NSNotFound || orderByLocation != NSNotFound)
     {
       if (batchable)
         {
-          [logger log: LogStandard: @"[DBSoap queryIdentify] LIMIT specifier incompatible with batch size > 1\n"];
+          [logger log: LogStandard: @"[DBSoap queryIdentify] option specifier incompatible with batch size > 1\n"];
           return;
         }
-      queryFirstPart = [queryString substringToIndex:limitLocation];
-      queryLimitPart = [queryString substringFromIndex:limitLocation];
+
+
+
+      if (orderByLocation != NSNotFound)
+        optionsLocation = orderByLocation;
+
+      if (limitLocation != NSNotFound)
+        {
+          if (orderByLocation != NSNotFound)
+            {
+              if  (orderByLocation > limitLocation)
+                {
+                  [logger log: LogStandard: @"[DBSoap queryIdentify] LIMIT specifier found before ORDER BY, ignoring\n"];
+                  optionsLocation = limitLocation;
+                }  
+            }
+          else
+            {
+              optionsLocation = limitLocation;
+            }
+        }
+      NSAssert(optionsLocation != NSNotFound, @"[DBSoap queryIdentify] optionsLocation can't be NSNotFound here");
+      queryFirstPart = [queryString substringToIndex:optionsLocation];
+      queryOptionsPart = [queryString substringFromIndex:optionsLocation];
+      [logger log: LogDebug: @"[DBSoap queryIdentify] Query Options: %@\n", queryOptionsPart];
     }
   else
     {
@@ -744,11 +816,11 @@
 	      [completeQuery appendString: @" )"];
 	    }
 
-          /* append LIMIT clause if present */
-          if (limitLocation != NSNotFound)
+          /* append options (ORDER BY, LIMIT) to clause if present */
+          if (optionsLocation != NSNotFound)
             {
               [completeQuery appendString: @" "];
-              [completeQuery appendString:queryLimitPart];
+              [completeQuery appendString:queryOptionsPart];
             }
 	  i++;
 	}
@@ -818,7 +890,7 @@
   insert an array of DBSObjects.<br>
   The objects in the array shall all be of the same type.
  */
-- (void)create :(NSString *)objectName fromArray:(NSMutableArray *)objects progressMonitor:(id<DBProgressProtocol>)p
+- (NSMutableArray *)create :(NSString *)objectName fromArray:(NSMutableArray *)objects progressMonitor:(id<DBProgressProtocol>)p
 {
   NSMutableDictionary   *headerDict;
   NSMutableDictionary   *sessionHeaderDict;
@@ -831,14 +903,13 @@
   DBSObject             *sObject;
   unsigned              batchCounter;
   NSUInteger            totalCounter;
+  NSMutableArray      *resultArray;
 
   if ([objects count] == 0)
-    return;
+    return nil;
 
   [p setMaximumValue: [objects count]];
   
-  upBatchSize = 1; // FIXME ########
-  
   /* prepare the header */
   sessionHeaderDict = [[NSMutableDictionary dictionaryWithCapacity: 2] retain];
   [sessionHeaderDict setObject: sessionId forKey: @"sessionId"];
@@ -854,6 +925,8 @@
   batchCounter = 0;
   totalCounter = 1;
   queryObjectsArray = [[NSMutableArray arrayWithCapacity: upBatchSize] retain];
+  resultArray = [[NSMutableArray arrayWithCapacity:1] retain];
+
   while ((sObject = [enumerator nextObject]))
   {
     unsigned            i;
@@ -864,14 +937,8 @@
     NSMutableDictionary *parmsDict;
     NSMutableDictionary *queryParmDict;
     NSDictionary        *result;
-    NSArray             *records;
-    NSDictionary        *record;
     NSDictionary        *queryFault;
-    NSString            *sizeStr;
-    int                 size;
-
-    
-    
+   
     NSLog(@"inner cycle: %d", batchCounter);
     sObj = [NSMutableDictionary dictionaryWithCapacity: 2];
     [sObj setObject: @"urn:partner.soap.sforce.com" forKey: GWSSOAPNamespaceURIKey];
@@ -892,7 +959,7 @@
 	NSString *keyName;
 
 	keyName = [fieldNames objectAtIndex:i];
-        [sObj setObject: [sObject fieldValue:keyName] forKey:keyName];
+        [sObj setObject: [sObject valueForField:keyName] forKey:keyName];
         [sObjKeyOrder addObject:keyName];
       }
     [sObj setObject: sObjKeyOrder forKey: GWSOrderKey];
@@ -941,54 +1008,70 @@
 	queryResult = [resultDict objectForKey:@"GWSCoderParameters"];
 	result = [queryResult objectForKey:@"result"];
 	[logger log: LogDebug: @"[DBSoap create] result: %@\n", result];
-
-	records = [result objectForKey:@"records"];
-	sizeStr = [result objectForKey:@"size"];
  
+        if (result != nil)
+          {
+            NSArray *results;
+            NSUInteger i;
 
-	if (sizeStr != nil)
-	  {
-	    int            i;
-	    int            j;
-	    int    batchSize;
-	    NSMutableArray *keys;
-	    NSMutableArray *set;
-      
-      
-	    size = [sizeStr intValue];
-	    batchSize = [records count];
-	    NSLog(@"Declared size is: %d", size);
-	    NSLog(@"records size is: %d", batchSize);
-      
-	    /* let's get the fields from the keys of the first record */
-	    record = [records objectAtIndex:0];
-	    keys = [NSMutableArray arrayWithArray:[record allKeys]];
-	    [keys removeObject:@"GWSCoderOrder"];
+            if (![result isKindOfClass:[NSArray class]])
+              {
+                NSLog(@"Single result. Repackaging into array");
+                results = [NSArray arrayWithObject:result];
+              }
+            else
+              {
+                results = (NSArray *)result;
+              }
 
-	    NSLog(@"keys: %@", keys);
-      
-	    set = [[NSMutableArray alloc] init];
-      
-	    /* now cycle all the records and read out the fields */
-	    for (i = 0; i < batchSize; i++)
-	      {
-		NSMutableArray *values;
-	  
-		record = [records objectAtIndex:i];
-		values = [NSMutableArray arrayWithCapacity:[keys count]];
-		for (j = 0; j < [keys count]; j++)
-		  {
-		    NSString *value;
-	      
-		    value = [record objectForKey:[keys objectAtIndex:j]];
-		    [values addObject:value];
-		  }
-		NSLog(@"%d: %@", i, values);
-		[set addObject:values];
-	      }
-	    /* we don't do yet anything useful with the results... */
-	    [set release];
+            for (i = 0; i < [results count]; i++)
+              {
+                NSString *objId;
+                NSString *successStr;
+                BOOL success;
+                NSString *message;
+                NSString *code;
+                NSDictionary *r;
+                NSDictionary *errors;
+                NSDictionary *rowDict;
+
+                r = [results objectAtIndex:i];
+                objId = [r objectForKey:@"id"];
+                successStr = [r objectForKey:@"success"];
+                success = NO;
+                if ([successStr isEqualToString:@"true"])
+                  success = YES;
+                code = nil;
+                message = nil;
+                errors = [r objectForKey:@"errors"];
+                if (errors != nil)
+                  {
+                    message = [errors objectForKey:@"message"];
+                    code = [errors objectForKey:@"statusCode"];
+                  }
+                NSLog(@"result: %@ -> %d, %@: %@ (%@)", objId, success, code, message, r);
+                if (success)
+                  {
+                    rowDict = [NSDictionary dictionaryWithObjectsAndKeys:
+                                              successStr, @"success",
+                                            objId, @"id",
+                                            @"", @"message",
+                                            @"", @"statusCode",
+                                            nil];
+                  }
+                else
+                  {
+                    rowDict = [NSDictionary dictionaryWithObjectsAndKeys:
+                                              successStr, @"success",
+                                            objId, @"id",
+                                            message, @"message",
+                                            code, @"statusCode",
+                                            nil];
+                  }
+                [resultArray addObject:rowDict];
+              }
 	  }
+
 	[logger log: LogDebug: @"[DBSoap create] reiniting cycle...\n"];
 	[p incrementCurrentValue: batchCounter+1];
 	[queryObjectsArray removeAllObjects];
@@ -1004,6 +1087,7 @@
   [queryObjectsArray release];
   [sessionHeaderDict release];
   [headerDict release];
+  return [resultArray autorelease];
 }
 
 
@@ -1013,7 +1097,7 @@
   </p>
   <p>The batch size sent is determined by the upBatchSize property of the class</p>
  */
-- (void)update :(NSString *)objectName fromArray:(NSMutableArray *)objects progressMonitor:(id<DBProgressProtocol>)p
+- (NSMutableArray *)update :(NSString *)objectName fromArray:(NSMutableArray *)objects progressMonitor:(id<DBProgressProtocol>)p
 {
   NSMutableDictionary   *headerDict;
   NSMutableDictionary   *sessionHeaderDict;
@@ -1025,14 +1109,13 @@
   unsigned              batchCounter;
   NSUInteger            totalCounter;
   NSMutableArray        *queryObjectsArray;
+  NSMutableArray        *resultArray;
 
   if ([objects count] == 0)
-    return;
+    return nil;
 
   [p setMaximumValue: [objects count]];
   
-  upBatchSize = 1; // FIXME ########
-
   /* prepare the header */
   sessionHeaderDict = [[NSMutableDictionary dictionaryWithCapacity: 2] retain];
   [sessionHeaderDict setObject: sessionId forKey: @"sessionId"];
@@ -1042,7 +1125,7 @@
   [headerDict setObject: sessionHeaderDict forKey: @"SessionHeader"];
   [headerDict setObject: GWSSOAPUseLiteral forKey: GWSSOAPUseKey];
   
-  [logger log: LogDebug: @"[DBSoap update] update objects array: %@\n", objects];
+  [logger log: LogDebug: @"[DBSoap update] update objects array size: %lu\n", (unsigned long)[objects count]];
   
   [p setCurrentDescription:@"Updating"];
   
@@ -1050,6 +1133,7 @@
   batchCounter = 0;
   totalCounter = 1;
   queryObjectsArray = [[NSMutableArray arrayWithCapacity: upBatchSize] retain];
+  resultArray = [[NSMutableArray arrayWithCapacity:1] retain];
   while ((sObject = [enumerator nextObject]))
   {
     unsigned int i;
@@ -1063,7 +1147,6 @@
     NSDictionary        *result;
     NSDictionary        *queryFault;
 
-    NSLog(@"inner cycle: %u", batchCounter);
     sObj = [NSMutableDictionary dictionaryWithCapacity: 2];
     [sObj setObject: @"urn:partner.soap.sforce.com" forKey: GWSSOAPNamespaceURIKey];
     sObjKeyOrder = [NSMutableArray arrayWithCapacity: 2];
@@ -1083,20 +1166,18 @@
 	NSString *keyName;
 
 	keyName = [fieldNames objectAtIndex:i];
-	[sObj setObject: [sObject fieldValue:keyName] forKey:keyName];
+	[sObj setObject: [sObject valueForField:keyName] forKey:keyName];
 	[sObjKeyOrder addObject:keyName];
       }
     [sObj setObject: sObjKeyOrder forKey: GWSOrderKey];
     [queryObjectsArray addObject: sObj];
-    NSLog(@"total counter = %lu of %lu", totalCounter, [objects count]);
+//    NSLog(@"total counter = %lu of %lu", totalCounter, [objects count]);
     if (batchCounter == upBatchSize-1 || totalCounter == [objects count])
       {
 	/* prepare the parameters */
 	queryParmDict = [NSMutableDictionary dictionaryWithCapacity: 2];
 	[queryParmDict setObject: @"urn:partner.soap.sforce.com" forKey: GWSSOAPNamespaceURIKey];
 
-	NSLog(@"queryObjectsArray count: %d of batchCounter %d", [queryObjectsArray count], batchCounter);
-
 	queryObjectsDict = [NSDictionary dictionaryWithObjectsAndKeys: queryObjectsArray, GWSSOAPValueKey, nil];
 
 	[queryParmDict setObject: queryObjectsDict forKey: @"sObjects"];
@@ -1119,7 +1200,6 @@
 	    NSString *faultCode;
 	    NSString *faultString;
 	    
-
 	    faultCode = [queryFault objectForKey:@"faultcode"];
 	    faultString = [queryFault objectForKey:@"faultstring"];
 	    [logger log: LogStandard: @"[DBSoap update] fault code: %@\n", faultCode];
@@ -1130,7 +1210,7 @@
 
 	queryResult = [resultDict objectForKey:@"GWSCoderParameters"];
 	result = [queryResult objectForKey:@"result"];
-	NSLog(@"query result: %@", result);
+	//NSLog(@"query result: %@", result);
 
 	if (result != nil)
 	  {
@@ -1139,7 +1219,6 @@
 
             if (![result isKindOfClass:[NSArray class]])
               {
-                NSLog(@"Single result. Repackaging into array");
                 results = [NSArray arrayWithObject:result];
               }
             else
@@ -1156,6 +1235,7 @@
                 NSString *code;
                 NSDictionary *r;
                 NSDictionary *errors;
+		NSDictionary *rowDict;
                 
                 r = [results objectAtIndex:i];
                 objId = [r objectForKey:@"id"];
@@ -1171,7 +1251,26 @@
                     message = [errors objectForKey:@"message"];
                     code = [errors objectForKey:@"statusCode"];
                   }
-                NSLog(@"result: %@ -> %d, %@: %@", objId, success, code, message);
+                //                NSLog(@"result: %@ -> %d, %@: %@ (%@)", objId, success, code, message, r);
+		if (success)
+		  {
+		    rowDict = [NSDictionary dictionaryWithObjectsAndKeys:
+					      successStr, @"success",
+					    objId, @"id",
+					    @"", @"message",
+					    @"", @"statusCode",
+					    nil];
+		  }
+		else
+		  {
+		    rowDict = [NSDictionary dictionaryWithObjectsAndKeys:
+					      successStr, @"success",
+					    objId, @"id",
+					    message, @"message",
+					    code, @"statusCode",
+					    nil];
+		  }
+		[resultArray addObject:rowDict];
               }
 	  }
 
@@ -1184,13 +1283,13 @@
 	batchCounter++;
       }
     totalCounter++;
-    NSLog(@"outer cycle.... %lu", totalCounter);
   } /* while: outer global object enumerator cycle */
   [logger log: LogDebug: @"[DBSoap update] outer cycle ended %lu\n", totalCounter];
 
   [queryObjectsArray release];
   [sessionHeaderDict release];
   [headerDict release];
+  return [resultArray autorelease];
 }
 
 
@@ -1329,55 +1428,6 @@
     [sObjectNamesList addObject: [[sObjectList objectAtIndex: i] name]];
 }
 
-- (void)describeSObject: (NSString *)objectType toWriter:(DBCVSWriter *)writer
-{
-  unsigned       i;
-  unsigned       size;
-  DBSObject      *object;
-  NSDictionary   *properties;
-  NSArray        *fields;
-  NSArray        *keys;
-  NSMutableArray *set;
-
-  
-  object = [self describeSObject: objectType];
-  fields = [object fieldNames];
-  size = [fields count];
-  
-  if (size < 1)
-    return;
-  
-  keys = [[object propertiesOfField: [fields objectAtIndex: 0]] allKeys];
-  [writer setFieldNames:[NSArray arrayWithArray:keys] andWriteIt:YES];
-  
-  set = [[NSMutableArray alloc] init];
-  
-  for (i = 0; i < size; i++)
-    {
-      NSMutableArray *values;
-      unsigned       j;
-      NSString       *field;
-      
-      field = [fields objectAtIndex: i];
-      properties = [object propertiesOfField: field];
-      values = [NSMutableArray arrayWithCapacity:[keys count]];
-      for (j = 0; j < [keys count]; j++)
-        {
-          id       obj;
-          id       value;
-          NSString *key;
-      
-          key = [keys objectAtIndex:j];
-          obj = [properties objectForKey: key];
-      
-          value = obj;
-          [values addObject:value];
-        }
-      [set addObject:values];
-    }
-  [writer writeDataSet:set];
-  [set release];
-}
 
 - (DBSObject *)describeSObject: (NSString *)objectType
 {
@@ -1390,13 +1440,15 @@
   NSDictionary          *result;
   NSDictionary          *queryFault;
   NSArray               *records;
+  NSArray               *recordTypeObjs;
   NSDictionary          *record;
   unsigned              i;
-  unsigned              size;
   NSMutableArray        *keys;
   DBSObject             *object;
   NSMutableDictionary   *propDict;
-
+  NSMutableArray        *rtArray;
+  NSMutableArray        *rtArray2;
+  NSMutableString       *queryString;
 
   /* prepare the header */
   sessionHeaderDict = [NSMutableDictionary dictionaryWithCapacity: 2];
@@ -1436,29 +1488,36 @@
       [logger log: LogStandard :@"[DBSoap describeSObject] exception code: %@\n", [fault objectForKey:@"exceptionCode"]];
       [logger log: LogStandard :@"[DBSoap describeSObject] exception message: %@\n", [fault objectForKey:@"exceptionMessage"]];
       [[NSException exceptionWithName:@"DBException" reason:[fault objectForKey:@"exceptionMessage"] userInfo:nil] raise];
+      return nil;
     }
 
   queryResult = [resultDict objectForKey:@"GWSCoderParameters"];
   result = [queryResult objectForKey:@"result"];
 
+  object = [[DBSObject alloc] init];
+
+  /* Extract Object Properties */
+  propDict = [NSMutableDictionary dictionaryWithCapacity: 1];
+  [propDict setValue:[result objectForKey:@"name"] forKey: @"name"];
+  [propDict setValue:[result objectForKey:@"label"] forKey: @"label"];
+  [propDict setValue:[result objectForKey:@"keyPrefix"] forKey: @"keyPrefix"];
+  [object setObjectProperties: propDict];
+
+
+  /* Extract Fields */
   records = [result objectForKey:@"fields"];
-  size = [records count];
 
   /* if we have only one element, put it in an array */
-  if (size == 1)
+  if ([records count] == 1)
     records = [NSArray arrayWithObject:records];
-  record = [records objectAtIndex:0];    
- 
+
+  record = [records objectAtIndex:0]; 
 
   keys = [NSMutableArray arrayWithArray:[record allKeys]];
   [keys removeObject:@"GWSCoderOrder"];
 
-  object = [[DBSObject alloc] init];
-  propDict = [NSMutableDictionary dictionaryWithCapacity: 1];
-  [propDict setValue: objectType forKey: @"name"];
-  [object setObjectProperties: propDict];
 
-  for (i = 0; i < size; i++)
+  for (i = 0; i < [records count]; i++)
     {
       NSMutableDictionary *props;
       NSString *fieldName;
@@ -1469,11 +1528,87 @@
       fieldName = [props objectForKey: @"name"];
       [object setProperties:[NSDictionary dictionaryWithDictionary: props] forField: fieldName];
     }
-    return [object autorelease];
+
+  /* Extract Record Types */
+  recordTypeObjs = [result objectForKey:@"recordTypeInfos"];
+ 
+  /* some objects don't have record-types at all, for others get additional information from RecordType */
+  if (recordTypeObjs)
+    {
+      [recordTypeObjs retain]; // we retain, since executing another query would otherwise clean the result
+
+      /* query record-type developer names with a subquery to RecordTypes */
+      queryString = [[NSMutableString alloc] init];
+      [queryString appendString:@"select Name, DeveloperName, Id from RecordType where SObjectType='"];
+      [queryString appendString: objectType];
+      [queryString appendString: @"'"];
+      NS_DURING
+        rtArray2 = [self queryFull:queryString queryAll:NO progressMonitor:nil];
+      NS_HANDLER
+        NSLog(@"Exception during record-type sub-query, %@", queryString);
+      rtArray2 = nil;
+      NS_ENDHANDLER
+        [queryString release];
+
+      /* if we have only one element, put it in an array */
+      if (![recordTypeObjs isKindOfClass:[NSArray class]])
+        {
+          [recordTypeObjs autorelease];
+          recordTypeObjs = [NSArray arrayWithObject:recordTypeObjs];
+          [recordTypeObjs retain];
+        }
+
+      rtArray = [NSMutableArray arrayWithCapacity: [recordTypeObjs count]];
+      for (i = 0; i < [recordTypeObjs count]; i++)
+        {
+          NSMutableDictionary *mDict;
+          NSString *devName;
+
+          record = [recordTypeObjs objectAtIndex:i];
+          mDict = [NSMutableDictionary dictionaryWithDictionary: record];
+          [mDict removeObjectForKey:@"GWSCoderOrder"];
+          NSLog(@"record-type from object: %@", mDict);
+          devName = nil;
+          /* we check for the master record type, for which the code is hardcoded by SF */
+          if ([[mDict objectForKey:@"recordTypeId"] isEqualToString:@"012000000000000AAA"])
+            {
+              devName = @"Master";
+            }
+          else
+            {
+              NSUInteger j;
+              NSString *rtId;
+
+              rtId = [mDict objectForKey:@"recordTypeId"];
+              for (j = 0; j < [rtArray2 count]; j++)
+                {
+                  DBSObject *so;
+
+                  so = [rtArray2 objectAtIndex: j];
+                  if ([[so sfId] isEqualToString:rtId])
+                    {
+                      devName = [so valueForField:@"DeveloperName"];
+                      NSLog(@"found: %@", devName);
+                    }
+                }
+            }
+          [rtArray addObject: mDict];
+
+          if (devName)
+            [mDict setObject:devName forKey:@"DeveloperName"];
+          else
+            NSLog(@"DBSoap: error, developer name for RecordTypeId %@ not found", [mDict objectForKey:@"Id"]);
+        }
+      [recordTypeObjs release];
+      NSLog(@"Record types: %@", rtArray);
+      [object setRecordTypes: [NSArray arrayWithArray: rtArray]];
+    }
+
+  return [object autorelease];
 }
 
 
-- (NSMutableArray *)delete :(NSArray *)objectIdArray
+- (NSMutableArray *)delete :(NSArray *)array progressMonitor:(id<DBProgressProtocol>)p;
 {
   NSMutableDictionary   *headerDict;
   NSMutableDictionary   *sessionHeaderDict;
@@ -1481,12 +1616,14 @@
   NSEnumerator          *enumerator;
   unsigned              batchCounter;
   NSMutableArray        *batchObjArray;
+  id                    objToDelete;
   NSString              *idStr;
 
-  if ([objectIdArray count] == 0)
+  if ([array count] == 0)
     return nil;
 
-  [logger log: LogDebug :@"[DBSoap delete] deleting %u objects...\n", [objectIdArray count]];
+  [logger log: LogDebug :@"[DBSoap delete] deleting %u objects...\n", [array count]];
+  [p setMaximumValue:[array count]];
 
   /* prepare the header */
   sessionHeaderDict = [NSMutableDictionary dictionaryWithCapacity: 2];
@@ -1497,7 +1634,7 @@
   [headerDict setObject: sessionHeaderDict forKey: @"SessionHeader"];
   [headerDict setObject: GWSSOAPUseLiteral forKey: GWSSOAPUseKey];
 
-  enumerator = [objectIdArray objectEnumerator];
+  enumerator = [array objectEnumerator];
   batchCounter = 0;
   batchObjArray = [[NSMutableArray arrayWithCapacity: MAX_BATCH_SIZE] retain];
   resultArray = [[NSMutableArray arrayWithCapacity:1] retain];
@@ -1512,7 +1649,14 @@
       NSDictionary          *queryFault;
       NSMutableArray        *queryObjectsDict;
 
-      idStr = [enumerator nextObject];
+      objToDelete = [enumerator nextObject];
+      if ([objToDelete isKindOfClass:[DBSObject class]])
+        {
+          idStr = [(DBSObject *)objToDelete sfId];
+        }
+      else
+        idStr = objToDelete;
+
       if (idStr)
 	{
 	  [batchObjArray addObject: idStr];
@@ -1521,7 +1665,7 @@
       /* did we fill a batch or did we reach the end? */
       if (batchCounter == MAX_BATCH_SIZE || !idStr)
 	{
-	  [logger log: LogDebug :@"[DBSoapNSLog delete ] batch obj-> %@\n", batchObjArray];
+          //	  [logger log: LogDebug :@"[DBSoapNSLog delete ] batch obj-> %@\n", batchObjArray];
 	  
 	  /* prepare the parameters */
 	  queryParmDict = [NSMutableDictionary dictionaryWithCapacity: 2];
@@ -1556,7 +1700,7 @@
   
 	  queryResult = [resultDict objectForKey:@"GWSCoderParameters"];
 	  result = [queryResult objectForKey:@"result"];
-	  NSLog(@"result: %@", result);
+          //	  NSLog(@"result: %@", result);
 
 	  if (result != nil)
 	    {
@@ -1608,11 +1752,10 @@
 		  [resultArray addObject:rowDict];
 		}
 	    }
-	  NSLog(@"reiniting batch....");
+          [p incrementCurrentValue:batchCounter];
 	  [batchObjArray removeAllObjects];
 	  batchCounter = 0;
 	} /* of batch */
-      NSLog(@"end of while loop...%@", idStr);
     }
   while (idStr);
 
@@ -1621,22 +1764,6 @@
   return [resultArray autorelease];
 }
 
-- (NSMutableArray *)deleteFromReader:(DBCVSReader *)reader
-{
-  NSMutableArray *objectsArray;
-  NSMutableArray *resultArray;
-
-  /* retrieve objects to delete */
-  // FIXME perhaps this copy is useless
-  objectsArray = [[NSMutableArray arrayWithArray:[reader readDataSet]] retain];
-  [logger log: LogDebug :@"[DBSoap deleteFromReader] objects to delete: %@\n", objectsArray];
-  NSLog(@"count of objects to delete: %u", [objectsArray count]);
-
-  resultArray = [self delete:objectsArray];
-  [objectsArray release];
-  return resultArray;
-}
-
 - (NSString *)identifyObjectById:(NSString *)sfId
 {
   NSString *devName;
@@ -1668,14 +1795,13 @@
       if ([[[tempObj objectProperties] objectForKey: @"keyPrefix"] isEqualToString: prefixToIdentify])
 	{
 	  name = [tempObj name];
-	  [logger log: LogDebug :@"[DBSoap identifyObjectById] we found: %@\n", name];
 	  found = YES;
 	}
     }
 
   if (found)
     {
-      [logger log: LogDebug :@"[DBSoap identifyObjectById] we found: %@\n", name];
+      [logger log: LogInformative :@"[DBSoap identifyObjectById] we found: %@\n", name];
       devName = [NSString stringWithString: name];
     }
   else
