@@ -1,26 +1,27 @@
 /*
-   Project: DataBasin
+  Project: DataBasin
 
-   Copyright (C) 2008-2013 Free Software Foundation
+  Copyright (C) 2008-2013 Free Software Foundation
 
-   Author: Riccardo Mottola
+  Author: Riccardo Mottola
 
-   Created: 2011-11-08 22:44:45 +0100 by multix
+  Created: 2011-11-08 22:44:45 +0100 by multix
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
- 
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
- 
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
- */
+  This library is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2 of the License, or (at your option) any later version.
+
+  This library is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Library General Public License for more details.
+
+  You should have received a copy of the GNU Lesser General Public
+  License along with this library; if not, write to the Free
+  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+  Boston, MA 02111 USA.
+*/
 
 #import <AppKit/AppKit.h>
 
@@ -146,14 +147,15 @@
   [p setEnd];
 }
 
-- (void)create :(NSString *)objectName fromReader:(DBCVSReader *)reader progressMonitor:(id<DBProgressProtocol>)p
+- (NSMutableArray *)create :(NSString *)objectName fromReader:(DBCVSReader *)reader progressMonitor:(id<DBProgressProtocol>)p
 {
-  NSEnumerator          *enumerator;
-  NSArray               *objectsArray;
-  NSArray               *fieldValues;
-  NSArray               *fieldNames;
-  int                   fieldCount;
-  NSMutableArray        *sObjectsArray;
+  NSEnumerator   *enumerator;
+  NSArray        *objectsArray;
+  NSArray        *fieldValues;
+  NSArray        *fieldNames;
+  NSUInteger     fieldCount;
+  NSMutableArray *sObjectsArray;
+  NSMutableArray *resultArray;
 
   /* retrieve objects to create */
   
@@ -180,8 +182,9 @@
     [sObj release];
   }
 
+  resultArray = nil;
   NS_DURING
-    [db create:objectName fromArray:sObjectsArray progressMonitor:p];
+    resultArray = [db create:objectName fromArray:sObjectsArray progressMonitor:p];
   NS_HANDLER
     [sObjectsArray release];
     [localException raise];
@@ -189,16 +192,18 @@
   [sObjectsArray release];
   [p setCurrentDescription:@"Done"];
   [p setEnd];
+  return resultArray;
 }
 
-- (void)update :(NSString *)objectName fromReader:(DBCVSReader *)reader progressMonitor:(id<DBProgressProtocol>)p
+- (NSMutableArray *)update :(NSString *)objectName fromReader:(DBCVSReader *)reader progressMonitor:(id<DBProgressProtocol>)p
 {
-  NSEnumerator          *enumerator;
-  NSArray               *objectsArray;
-  NSArray               *fieldValues;
-  NSArray               *fieldNames;
-  int                   fieldCount;
-  NSMutableArray        *sObjectsArray;
+  NSEnumerator   *enumerator;
+  NSArray        *objectsArray;
+  NSArray        *fieldValues;
+  NSArray        *fieldNames;
+  NSUInteger     fieldCount;
+  NSMutableArray *sObjectsArray;
+  NSMutableArray *resultArray;
 
   /* retrieve objects to update */
   [p reset];
@@ -227,8 +232,9 @@
     [sObj release];
   }
 
+  resultArray = nil;
   NS_DURING
-    [db update:objectName fromArray:sObjectsArray progressMonitor:p];
+    resultArray = [db update:objectName fromArray:sObjectsArray progressMonitor:p];
   NS_HANDLER
     [sObjectsArray release];
     [localException raise];
@@ -237,12 +243,13 @@
   [sObjectsArray release];
   [p setCurrentDescription:@"Done"];
   [p setEnd];
+  return resultArray;
 }
 
 - (void)describeSObject: (NSString *)objectType toWriter:(DBCVSWriter *)writer
 {
-  int            i;
-  int            size;
+  NSUInteger      i;
+  NSUInteger     size;
   DBSObject      *object;
   NSDictionary   *properties;
   NSArray        *fields;
@@ -265,7 +272,7 @@
   for (i = 0; i < size; i++)
     {
       NSMutableArray *values;
-      int            j;
+      NSUInteger      j;
       NSString       *field;
       
       field = [fields objectAtIndex: i];
@@ -291,7 +298,7 @@
 
 
 
-- (NSMutableArray *)deleteFromReader:(DBCVSReader *)reader
+- (NSMutableArray *)deleteFromReader:(DBCVSReader *)reader progressMonitor:(id<DBProgressProtocol>)p
 {
   NSMutableArray *objectsArray;
   NSMutableArray *resultArray;
@@ -304,7 +311,7 @@
 
   resultArray = nil;
   NS_DURING
-    resultArray = [db delete:objectsArray];
+    resultArray = [db delete:objectsArray progressMonitor:p];
   NS_HANDLER
     [objectsArray release];
     [localException raise];
